==> kazehakase-0.0.9.tar.gz: kazehakase-0.0.9/src/bookmarks/kz-remote-bookmark.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <gtk/gtk.h>
#include <unistd.h>

#include "kz-remote-bookmark.h"
#include "kz-rss.h"
#include "kz-lirs.h"
#include "kz-hinadi.h"
#include "gobject-utils.h"
#include "intl.h"
#include "kazehakase.h"
#include "kz-profile.h"
#include "kz-http.h"

enum {
	START_SIGNAL,
	UPDATE_START_SIGNAL,
	UPDATE_COMPLETED_SIGNAL,
	LAST_SIGNAL
};

enum {
	PROP_0,
	PROP_TITLE,
	PROP_URI,
	PROP_INTERVAL,
};

typedef enum {
	KZ_REMOTE_BOOKMARK_RSS,
	KZ_REMOTE_BOOKMARK_LIRS,
	KZ_REMOTE_BOOKMARK_HINADI,
	KZ_REMOTE_BOOKMARK_XBEL,
} KzRemoteBookmarkType;

static void     kz_remote_bookmark_class_init      (KzRemoteBookmarkClass *klass);
static void     kz_remote_bookmark_init            (KzRemoteBookmark *rbookmark);
static GObject *kz_remote_bookmark_constructor     (GType                  type,
						    guint                  n_props,
						    GObjectConstructParam *props);
static void     kz_remote_bookmark_set_property    (GObject *object,
						    guint prop_id,
						    const GValue *value,
						    GParamSpec *pspec);
static void     kz_remote_bookmark_get_property    (GObject *object,
						    guint prop_id,
						    GValue *value,
						    GParamSpec *pspec);

static void kz_remote_bookmark_file_fetch          (KzRemoteBookmark *rbookmark);
/* static void kz_remote_bookmark_real_items_update   (KzRemoteBookmark *rbookmark); */

static void kz_remote_bookmark_update_complete     (KzHTTP *http, KzRemoteBookmark *rbookmark);

static KzRemoteBookmarkType kz_remote_bookmark_detect_type(KzRemoteBookmark *rbookmark, gchar *buf);

static KzBookmarkClass *parent_class = NULL;
static gint kz_remote_bookmark_signals[LAST_SIGNAL] = {0};

KZ_OBJECT_GET_TYPE(kz_remote_bookmark, "KzRemoteBookmark", KzRemoteBookmark,
		   kz_remote_bookmark_class_init, kz_remote_bookmark_init,
		   KZ_TYPE_BOOKMARK)


static void
kz_remote_bookmark_class_init (KzRemoteBookmarkClass *klass)
{
	GObjectClass *object_class;

	parent_class = g_type_class_peek_parent (klass);
	object_class = (GObjectClass *) klass;

	object_class->constructor  = kz_remote_bookmark_constructor;
	object_class->dispose      = kz_remote_bookmark_dispose;
	object_class->set_property = kz_remote_bookmark_set_property;
	object_class->get_property = kz_remote_bookmark_get_property;
	
	klass->update_start         = NULL;
	klass->update_completed     = NULL;
	
	g_object_class_install_property(
		object_class,
		 PROP_INTERVAL,
		 g_param_spec_uint(
			 "interval",
			 _("Interval"),
			 _("Update interval"),
			 0,
			 G_MAXUINT,
			 0,
			 G_PARAM_READWRITE));

	kz_remote_bookmark_signals[UPDATE_START_SIGNAL]
		= g_signal_new ("update_start",
				G_TYPE_FROM_CLASS (klass),
				G_SIGNAL_RUN_LAST,
				G_STRUCT_OFFSET (KzRemoteBookmarkClass, update_start),
				NULL, NULL,
				g_cclosure_marshal_VOID__VOID,
				G_TYPE_NONE, 0);

	kz_remote_bookmark_signals[UPDATE_COMPLETED_SIGNAL]
		= g_signal_new ("update_completed",
				G_TYPE_FROM_CLASS (klass),
				G_SIGNAL_RUN_LAST,
				G_STRUCT_OFFSET (KzRemoteBookmarkClass, update_completed),
				NULL, NULL,
				g_cclosure_marshal_VOID__VOID,
				G_TYPE_NONE, 0);
}


static void
kz_remote_bookmark_init (KzRemoteBookmark *rbookmark)
{
	rbookmark->timer_id        = 0;
	rbookmark->update_interval = 0;

	rbookmark->state = KZ_REMOTE_BOOKMARK_NORMAL;
}


static GObject*
kz_remote_bookmark_constructor (GType                  type,
				guint                  n_props,
				GObjectConstructParam *props)
{
	GObject *object;
	GObjectClass *klass = G_OBJECT_CLASS(parent_class);

	object = klass->constructor(type, n_props, props);

	g_object_set(object, "type", KZ_BOOKMARK_FOLDER, NULL);

	return object;
}


void
kz_remote_bookmark_dispose (GObject *object)
{
	KzRemoteBookmark *rbookmark;

	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(object));

	rbookmark = KZ_REMOTE_BOOKMARK(object);

	if (rbookmark->timer_id)
		gtk_timeout_remove(rbookmark->timer_id);
	rbookmark->timer_id = 0;

	kz_bookmark_remove_all(KZ_BOOKMARK(rbookmark));

	if (G_OBJECT_CLASS (parent_class)->dispose)
		G_OBJECT_CLASS (parent_class)->dispose(object);
}

static void
kz_remote_bookmark_set_property (GObject *object,
				 guint prop_id,
				 const GValue *value,
				 GParamSpec *pspec)
{
	KzRemoteBookmark *rbookmark = KZ_REMOTE_BOOKMARK(object);

	switch (prop_id)
	{
	case PROP_INTERVAL:
		rbookmark->update_interval = g_value_get_uint(value);

		if (rbookmark->timer_id)
			gtk_timeout_remove(rbookmark->timer_id);
		rbookmark->timer_id = 0;

		if (rbookmark->update_interval > 0)
			rbookmark->timer_id
				= gtk_timeout_add(60000 * rbookmark->update_interval,
						  (void *)&kz_remote_bookmark_update, rbookmark);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}


static void
kz_remote_bookmark_get_property (GObject *object,
				 guint prop_id,
				 GValue *value,
				 GParamSpec *pspec)
{
	KzRemoteBookmark *rbookmark = KZ_REMOTE_BOOKMARK(object);

	switch (prop_id)
	{
	case PROP_INTERVAL:
		g_value_set_uint(value, rbookmark->update_interval);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}


KzRemoteBookmark *
kz_remote_bookmark_new (gchar *location, gchar *title, guint update_interval)
{
	KzRemoteBookmark *rbookmark;

	rbookmark = KZ_REMOTE_BOOKMARK(g_object_new(KZ_TYPE_REMOTE_BOOKMARK,
						    "location", location,
						    "title",    title,
						    "interval", update_interval,
						    NULL));
	kz_remote_bookmark_update(rbookmark);

	return rbookmark;
}


static void
kz_remote_bookmark_file_fetch(KzRemoteBookmark *rbookmark)
{
	KzHTTP *http;
	const gchar *uri;

	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	uri = kz_bookmark_get_location(KZ_BOOKMARK(rbookmark));
	g_return_if_fail(uri && *uri);;

	http = kz_http_new(uri);
	g_signal_connect(G_OBJECT(http), "fetch_completed",
			 G_CALLBACK(kz_remote_bookmark_update_complete), rbookmark);
	g_signal_connect(G_OBJECT(http), "connect_error",
			 G_CALLBACK(kz_remote_bookmark_update_complete), rbookmark);
	kz_http_file_fetch(http);
}


static KzRemoteBookmarkType
kz_remote_bookmark_detect_type(KzRemoteBookmark *rbookmark, gchar *buf)
{
	KzRemoteBookmarkType type;

	type = KZ_REMOTE_BOOKMARK_RSS;
	
	switch (buf[0])
	{
	 case '<':
		if (strncmp(buf, "<?xml", 5) == 0)
		{
			gchar *pos = buf;
			while (pos && pos[1] != '!')
				pos = strchr(pos + 1, '<');
			if (pos)
			{
				if (strncmp(pos, "<rdf",4) == 0)
					type = KZ_REMOTE_BOOKMARK_RSS;
				else if (strncmp(pos, "<xbel", 5) == 0)
					type = KZ_REMOTE_BOOKMARK_XBEL;
			}
		}
		break;
	 case 'L':
		if (strncmp(buf, "LIRS", 4) == 0)
			type = KZ_REMOTE_BOOKMARK_LIRS;
		break;
	 case 'H':
		if (strncmp(buf, "HINA", 4) == 0)
			type = KZ_REMOTE_BOOKMARK_HINADI;
		break;
	 default:
		type = KZ_REMOTE_BOOKMARK_LIRS;
		break;
	}
	return type;
}


static gboolean
idle_update_complete(gpointer data)
{
	KzHTTP *http = data;

	g_object_unref(G_OBJECT(http));

	return FALSE;
}


static void
kz_remote_bookmark_update_complete(KzHTTP *http, KzRemoteBookmark *rbookmark)
{	
	KzRemoteBookmarkType type;

	if (http->bytes_loaded != 0)
	{
		type = kz_remote_bookmark_detect_type(rbookmark, http->buf->str);
		switch (type)
		{
		 case KZ_REMOTE_BOOKMARK_RSS:
			kz_rss_parse_from_string(rbookmark,
						 http->buf->str,
						 http->bytes_loaded,
						 NULL);
			break;
		 case KZ_REMOTE_BOOKMARK_LIRS:
			kz_lirs_parse_from_string(rbookmark,
						  http->buf->str,
						  http->bytes_loaded,
						  NULL);
			break;
		 case KZ_REMOTE_BOOKMARK_HINADI:
			kz_hinadi_parse_from_string(rbookmark,
						    http->buf->str,
						    http->bytes_loaded,
						    NULL);
			break;
		 default:
			break;
		}
	}
	
	g_signal_handlers_disconnect_by_func(G_OBJECT(http),
					     G_CALLBACK(kz_remote_bookmark_update_complete),
					     rbookmark);
	kz_http_dispose(G_OBJECT(http));
	rbookmark->state = KZ_REMOTE_BOOKMARK_NORMAL;
	g_signal_emit(G_OBJECT (rbookmark),
		      kz_remote_bookmark_signals[UPDATE_COMPLETED_SIGNAL],
		      0);

	gtk_idle_add(idle_update_complete, http);
}

static void 
kz_remote_bookmark_items_update (KzRemoteBookmark *rbookmark)
{
	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	kz_bookmark_remove_all(KZ_BOOKMARK(rbookmark));
	kz_remote_bookmark_file_fetch(rbookmark);
}


gboolean
kz_remote_bookmark_update (KzRemoteBookmark *rbookmark)
{
	if (rbookmark->state == KZ_REMOTE_BOOKMARK_LOADING)
		return TRUE;
	rbookmark->state = KZ_REMOTE_BOOKMARK_LOADING;

	g_signal_emit(G_OBJECT (rbookmark),
		      kz_remote_bookmark_signals[UPDATE_START_SIGNAL],
		      0);
	kz_remote_bookmark_items_update(rbookmark);
	return TRUE;
}


void
kz_remote_bookmark_force_update (KzRemoteBookmark *rbookmark)
{
	if (rbookmark->state == KZ_REMOTE_BOOKMARK_LOADING)
		return;
	kz_remote_bookmark_update(rbookmark);
}


void
kz_remote_bookmark_set_interval (KzRemoteBookmark *rbookmark, guint interval)
{
	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	g_object_set(G_OBJECT(rbookmark), "interval", interval);
}

==> kazehakase-0.0.9.1.tar.gz: kazehakase-0.0.9.1/src/bookmarks/kz-remote-bookmark.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <gtk/gtk.h>
#include <unistd.h>

#include "kz-remote-bookmark.h"
#include "kz-rss.h"
#include "kz-lirs.h"
#include "kz-hinadi.h"
#include "gobject-utils.h"
#include "intl.h"
#include "kazehakase.h"
#include "kz-profile.h"
#include "kz-downloader.h"

enum {
	START_SIGNAL,
	UPDATE_START_SIGNAL,
	UPDATE_COMPLETED_SIGNAL,
	LAST_SIGNAL
};

enum {
	PROP_0,
	PROP_TITLE,
	PROP_URI,
	PROP_INTERVAL,
};

typedef enum {
	KZ_REMOTE_BOOKMARK_RSS,
	KZ_REMOTE_BOOKMARK_LIRS,
	KZ_REMOTE_BOOKMARK_HINADI,
	KZ_REMOTE_BOOKMARK_XBEL,
} KzRemoteBookmarkType;

static void     kz_remote_bookmark_class_init      (KzRemoteBookmarkClass *klass);
static void     kz_remote_bookmark_init            (KzRemoteBookmark *rbookmark);
static GObject *kz_remote_bookmark_constructor     (GType                  type,
						    guint                  n_props,
						    GObjectConstructParam *props);
static void     kz_remote_bookmark_set_property    (GObject *object,
						    guint prop_id,
						    const GValue *value,
						    GParamSpec *pspec);
static void     kz_remote_bookmark_get_property    (GObject *object,
						    guint prop_id,
						    GValue *value,
						    GParamSpec *pspec);

static void kz_remote_bookmark_file_fetch          (KzRemoteBookmark *rbookmark);
/* static void kz_remote_bookmark_real_items_update   (KzRemoteBookmark *rbookmark); */

static void kz_remote_bookmark_update_complete     (KzDownloader *dl, KzRemoteBookmark *rbookmark);

static KzRemoteBookmarkType kz_remote_bookmark_detect_type(KzRemoteBookmark *rbookmark, gchar *buf);

static KzBookmarkClass *parent_class = NULL;
static gint kz_remote_bookmark_signals[LAST_SIGNAL] = {0};

KZ_OBJECT_GET_TYPE(kz_remote_bookmark, "KzRemoteBookmark", KzRemoteBookmark,
		   kz_remote_bookmark_class_init, kz_remote_bookmark_init,
		   KZ_TYPE_BOOKMARK)


static void
kz_remote_bookmark_class_init (KzRemoteBookmarkClass *klass)
{
	GObjectClass *object_class;

	parent_class = g_type_class_peek_parent (klass);
	object_class = (GObjectClass *) klass;

	object_class->constructor  = kz_remote_bookmark_constructor;
	object_class->dispose      = kz_remote_bookmark_dispose;
	object_class->set_property = kz_remote_bookmark_set_property;
	object_class->get_property = kz_remote_bookmark_get_property;
	
	klass->update_start         = NULL;
	klass->update_completed     = NULL;
	
	g_object_class_install_property(
		object_class,
		 PROP_INTERVAL,
		 g_param_spec_uint(
			 "interval",
			 _("Interval"),
			 _("Update interval"),
			 0,
			 G_MAXUINT,
			 0,
			 G_PARAM_READWRITE));

	kz_remote_bookmark_signals[UPDATE_START_SIGNAL]
		= g_signal_new ("update_start",
				G_TYPE_FROM_CLASS (klass),
				G_SIGNAL_RUN_LAST,
				G_STRUCT_OFFSET (KzRemoteBookmarkClass, update_start),
				NULL, NULL,
				g_cclosure_marshal_VOID__VOID,
				G_TYPE_NONE, 0);

	kz_remote_bookmark_signals[UPDATE_COMPLETED_SIGNAL]
		= g_signal_new ("update_completed",
				G_TYPE_FROM_CLASS (klass),
				G_SIGNAL_RUN_LAST,
				G_STRUCT_OFFSET (KzRemoteBookmarkClass, update_completed),
				NULL, NULL,
				g_cclosure_marshal_VOID__VOID,
				G_TYPE_NONE, 0);
}


static void
kz_remote_bookmark_init (KzRemoteBookmark *rbookmark)
{
	rbookmark->timer_id        = 0;
	rbookmark->update_interval = 0;

	rbookmark->state = KZ_REMOTE_BOOKMARK_NORMAL;
}


static GObject*
kz_remote_bookmark_constructor (GType                  type,
				guint                  n_props,
				GObjectConstructParam *props)
{
	GObject *object;
	GObjectClass *klass = G_OBJECT_CLASS(parent_class);

	object = klass->constructor(type, n_props, props);

	g_object_set(object, "type", KZ_BOOKMARK_FOLDER, NULL);

	return object;
}


void
kz_remote_bookmark_dispose (GObject *object)
{
	KzRemoteBookmark *rbookmark;

	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(object));

	rbookmark = KZ_REMOTE_BOOKMARK(object);

	if (rbookmark->timer_id)
		gtk_timeout_remove(rbookmark->timer_id);
	rbookmark->timer_id = 0;

	kz_bookmark_remove_all(KZ_BOOKMARK(rbookmark));

	if (G_OBJECT_CLASS (parent_class)->dispose)
		G_OBJECT_CLASS (parent_class)->dispose(object);
}

static void
kz_remote_bookmark_set_property (GObject *object,
				 guint prop_id,
				 const GValue *value,
				 GParamSpec *pspec)
{
	KzRemoteBookmark *rbookmark = KZ_REMOTE_BOOKMARK(object);

	switch (prop_id)
	{
	case PROP_INTERVAL:
		rbookmark->update_interval = g_value_get_uint(value);

		if (rbookmark->timer_id)
			gtk_timeout_remove(rbookmark->timer_id);
		rbookmark->timer_id = 0;

		if (rbookmark->update_interval > 0)
			rbookmark->timer_id
				= gtk_timeout_add(60000 * rbookmark->update_interval,
						  (void *)&kz_remote_bookmark_update, rbookmark);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}


static void
kz_remote_bookmark_get_property (GObject *object,
				 guint prop_id,
				 GValue *value,
				 GParamSpec *pspec)
{
	KzRemoteBookmark *rbookmark = KZ_REMOTE_BOOKMARK(object);

	switch (prop_id)
	{
	case PROP_INTERVAL:
		g_value_set_uint(value, rbookmark->update_interval);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
		break;
	}
}


KzRemoteBookmark *
kz_remote_bookmark_new (gchar *location, gchar *title, guint update_interval)
{
	KzRemoteBookmark *rbookmark;

	rbookmark = KZ_REMOTE_BOOKMARK(g_object_new(KZ_TYPE_REMOTE_BOOKMARK,
						    "location", location,
						    "title",    title,
						    "interval", update_interval,
						    NULL));
	kz_remote_bookmark_update(rbookmark);

	return rbookmark;
}


static void
kz_remote_bookmark_file_fetch(KzRemoteBookmark *rbookmark)
{
	KzDownloader *dl;
	const gchar *uri;

	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	uri = kz_bookmark_get_location(KZ_BOOKMARK(rbookmark));
	g_return_if_fail(uri && *uri);;

	dl = kz_downloader_new(uri);
	g_signal_connect(G_OBJECT(dl), "completed",
			 G_CALLBACK(kz_remote_bookmark_update_complete), rbookmark);
	g_signal_connect(G_OBJECT(dl), "error",
			 G_CALLBACK(kz_remote_bookmark_update_complete), rbookmark);
	kz_downloader_to_string(dl);
}


static KzRemoteBookmarkType
kz_remote_bookmark_detect_type(KzRemoteBookmark *rbookmark, gchar *buf)
{
	KzRemoteBookmarkType type;

	type = KZ_REMOTE_BOOKMARK_RSS;
	
	switch (buf[0])
	{
	 case '<':
		if (strncmp(buf, "<?xml", 5) == 0)
		{
			gchar *pos = buf;
			while (pos && pos[1] != '!')
				pos = strchr(pos + 1, '<');
			if (pos)
			{
				if (strncmp(pos, "<rdf",4) == 0)
					type = KZ_REMOTE_BOOKMARK_RSS;
				else if (strncmp(pos, "<xbel", 5) == 0)
					type = KZ_REMOTE_BOOKMARK_XBEL;
			}
		}
		break;
	 case 'L':
		if (strncmp(buf, "LIRS", 4) == 0)
			type = KZ_REMOTE_BOOKMARK_LIRS;
		break;
	 case 'H':
		if (strncmp(buf, "HINA", 4) == 0)
			type = KZ_REMOTE_BOOKMARK_HINADI;
		break;
	 default:
		type = KZ_REMOTE_BOOKMARK_RSS;
		break;
	}
	return type;
}


static gboolean
idle_update_complete(gpointer data)
{
	KzDownloader *dl = data;

	g_object_unref(G_OBJECT(dl));

	return FALSE;
}


static void
kz_remote_bookmark_update_complete(KzDownloader *dl, KzRemoteBookmark *rbookmark)
{	
	KzRemoteBookmarkType type;
	gchar *buf;
	guint size;

	buf  = kz_downloader_get_string(dl);
	size = kz_downloader_get_current_progress(dl);
	
	if (size != 0)
	{
		type = kz_remote_bookmark_detect_type(rbookmark, buf);
		switch (type)
		{
		 case KZ_REMOTE_BOOKMARK_RSS:
			kz_rss_parse_from_string(rbookmark,
						 buf,
						 size,
						 NULL);
			break;
		 case KZ_REMOTE_BOOKMARK_LIRS:
			kz_lirs_parse_from_string(rbookmark,
						  buf,
						  size,
						  NULL);
			break;
		 case KZ_REMOTE_BOOKMARK_HINADI:
			kz_hinadi_parse_from_string(rbookmark,
						    buf,
						    size,
						    NULL);
			break;
		 default:
			break;
		}
	}
	
	g_signal_handlers_disconnect_by_func(G_OBJECT(dl),
					     G_CALLBACK(kz_remote_bookmark_update_complete),
					     rbookmark);
	rbookmark->state = KZ_REMOTE_BOOKMARK_NORMAL;
	g_signal_emit(G_OBJECT (rbookmark),
		      kz_remote_bookmark_signals[UPDATE_COMPLETED_SIGNAL],
		      0);

	gtk_idle_add(idle_update_complete, dl);
}

static void 
kz_remote_bookmark_items_update (KzRemoteBookmark *rbookmark)
{
	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	kz_bookmark_remove_all(KZ_BOOKMARK(rbookmark));
	kz_remote_bookmark_file_fetch(rbookmark);
}


gboolean
kz_remote_bookmark_update (KzRemoteBookmark *rbookmark)
{
	if (rbookmark->state == KZ_REMOTE_BOOKMARK_LOADING)
		return TRUE;
	rbookmark->state = KZ_REMOTE_BOOKMARK_LOADING;

	g_signal_emit(G_OBJECT (rbookmark),
		      kz_remote_bookmark_signals[UPDATE_START_SIGNAL],
		      0);
	kz_remote_bookmark_items_update(rbookmark);
	return TRUE;
}


void
kz_remote_bookmark_force_update (KzRemoteBookmark *rbookmark)
{
	if (rbookmark->state == KZ_REMOTE_BOOKMARK_LOADING)
		return;
	kz_remote_bookmark_update(rbookmark);
}


void
kz_remote_bookmark_set_interval (KzRemoteBookmark *rbookmark, guint interval)
{
	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));

	g_object_set(G_OBJECT(rbookmark), "interval", interval);
}
